--[[
Most handlers there are dirty and hackish and should better be implemented in 
the addon themselves.

They all register a CONFIGMODE_CALLBACKS with the same as the addon except 
when such callback already exists. So it will NOT override addon-defined 
callback.

If the addon is not yet loaded but could be loaded later, a listener 
is set up to wait until the addon is loaded.
]]

-- Simple helper: return true if the addon exist and can be loaded
local function IsLoadable(name)
	local exists, _, _, enabled, loadable = GetAddOnInfo(name)
	return exists and enabled and loadable and true
end

local demos = {}

-- ag_UnitFrames
function demos.ag_UnitFrames()
	if CONFIGMODE_CALLBACKS.ag_UnitFrames then return end
	local aUF = LibStub("AceAddon-3.0"):GetAddon("ag_UnitFrames")
	local wasLocked
	function CONFIGMODE_CALLBACKS.ag_UnitFrames(action, mode)
		if action == 'GETMODES' then
			return 'party', 'raid'
		end
		LoadAddOn('ag_Options')
		if not aUF.UpdateSetupMode then
			return
		end
		if action == 'ON' then
			wasLocked = aUF.db.profile.Locked
			aUF.db.profile.Locked = false
			if mode == "raid" then
				aUF:UpdateSetupMode("raid")
			else
				aUF:UpdateSetupMode("party")
			end
		elseif action == 'OFF' then
			aUF:UpdateSetupMode('off')
			if wasLocked then
				aUF.db.profile.Locked = true
				wasLocked = nil
			end
		end
	end
end

-- Dominos
function demos.Dominos()
	if CONFIGMODE_CALLBACKS.Dominos then return end
	local Dominos = LibStub("AceAddon-3.0"):GetAddon("Dominos")
	local wasLocked
	function CONFIGMODE_CALLBACKS.Dominos(action)
		if action == 'ON' and Dominos:Locked() then
			Dominos:SetLock(false)
			wasLocked = true
		elseif action == 'OFF' and wasLocked then
			Dominos:SetLock(true)
			wasLocked = nil
		end
	end
end

-- SexyMap
function demos.SexyMap()
	if CONFIGMODE_CALLBACKS.SexyMap then return end
	local SexyMap = LibStub("AceAddon-3.0"):GetAddon("SexyMap")
	local General = SexyMap:GetModule('General')
	function CONFIGMODE_CALLBACKS.SexyMap(action)
		if action == 'ON' then
			General:SetLock(false)
		elseif action == 'OFF' then
			General:SetLock(true)
		end
	end
end

-- Quartz
function demos.Quartz()
	if CONFIGMODE_CALLBACKS.Quartz then return end
	local Quartz = _G.Quartz
	local qmodules = {
		Player = 'lock',
		Focus = 'lock',
		Target = 'lock',
		Pet = 'lock',
		Mirror = 'mirrorlock',
	}
	local toggled = {}
	function CONFIGMODE_CALLBACKS.Quartz(action)
		local options = Quartz.options.args
		if action == 'ON' then
			for module,optName in pairs(qmodules) do
				local opt = options[module] and options[module].args and options[module].args[optName]
				if opt and opt.get() then
					toggled[module] = optName
					opt.set(false)
				end
			end
		elseif action == 'OFF' then
			for module,optName in pairs(toggled) do
				local opt = options[module] and options[module].args and options[module].args[optName]
				opt.set(true)
			end
			wipe(toggled)
		end
	end
end

-- BigWigs
function demos.BigWigs()
	if CONFIGMODE_CALLBACKS.BigWigs then return end
	local toggled
	function CONFIGMODE_CALLBACKS.BigWigs(action)
		if action == "ON" then
			if BigWigs or LoadAddOn("BigWigs_Core") or LoadAddOn("BigWigs") then
				if not BigWigs:IsEnabled() then
					BigWigs:Enable()
					toggled = true
				end
				if not BigWigsOptions then
					LoadAddOn('BigWigs_Options')
				end
				BigWigs:SendMessage("BigWigs_StartConfigureMode")
			end
		elseif BigWigs and action == "OFF" then
			BigWigs:SendMessage("BigWigs_StopConfigureMode")
			if toggled then
				BigWigs:Disable()
				toggled = nil
			end
		end
	end
end

-- Baggins
function demos.Baggins()
	if CONFIGMODE_CALLBACKS.Baggins then return end
	local hasShownPlacement
	local wasLocked
	function CONFIGMODE_CALLBACKS.Baggins(action)
		if Baggins.db.profile.layout == "auto" then
			if action == "ON" then
				if not BagginsBagPlacement or not BagginsBagPlacement:IsShown() then
					Baggins:ShowPlacementFrame() 
					hasShownPlacement = true
				end
			elseif action == "OFF" and hasShownPlacement then
				BagginsBagPlacement:Hide()
			end
		else
			if action == "ON" then
				wasLocked = Baggins.db.profile.lock
				Baggins.db.profile.lock = false
			elseif action == "OFF" and wasLocked then
				Baggins.db.profile.lock = true
				wasLocked = nil
			end
		end		
	end
end

-- Auracle
function demos.Auracle()
	if CONFIGMODE_CALLBACKS.Auracle then return end
	function CONFIGMODE_CALLBACKS.Auracle(action)
		if action == 'ON' then
			Auracle:UnlockWindows()
			enabled = true
		elseif action == 'OFF' and enabled then
			Auracle:LockWindows()
		end
	end
end

-- Gladius
function demos.Gladius()
	if CONFIGMODE_CALLBACKS.Gladius then return end
	local wasLocked
	function CONFIGMODE_CALLBACKS.Gladius(action, mode)
		if action == 'ON' then
			wasLocked = Gladius.db.profile.locked
			Gladius.db.profile.locked = false
			
			local bracket = tonumber(tostring(mode):sub(1,1)) or 5
			if bracket ~= Gladius.currentBracket then
				Gladius:ToggleFrame(bracket)
			else
				Gladius:UpdateFrame()
			end
		elseif action == 'OFF' then
			if wasLocked then
				Gladius.db.profile.locked = true
				wasLocked = false
			end
			Gladius:HideFrame()
			
		elseif action == "GETMODES" then
			return "5vs5", "3vs3", "2vs2"
		end
	end
end

local eventFrame = CreateFrame("Frame")
eventFrame:SetScript('OnEvent', function(self, event, ...) self[event](self, event, ...) end)

function eventFrame:VARIABLES_LOADED()
	self:UnregisterEvent('VARIABLES_LOADED')
	self.VARIABLES_LOADED =  nil
	
	CONFIGMODE_CALLBACKS = CONFIGMODE_CALLBACKS or {}
	
	-- Immediatly register handlers for already loaded addons
	local pending
	for name, callback in pairs(demos) do
		if IsAddOnLoaded(name) then
			local ok, err = pcall(callback)
			if not ok then geterrorhandler()(err) end
		elseif IsLoadable(name) then
			if not pending then
				pending = {}
			end
			pending[name:lower()] = callback
		end
	end
	demos = nil

	if pending then
		-- Listen to ADDON_LOADED to register pending handlers 
		self:RegisterEvent('ADDON_LOADED')
		function self:ADDON_LOADED(_, name)
			local callback = pending[name:lower()]
			if callback then
				local ok, err = pcall(callback)
				if not ok then geterrorhandler()(err) end
				pending[name:lower()] = nil
			end
			if not next(pending) then
				-- Nothing more to do, clean up
				pending = nil
				self:UnregisterEvent('ADDON_LOADED')
				self.ADDON_LOADDED = nil				
			end
		end
	end	
end

if IsLoggedIn() then
	eventFrame:VARIABLES_LOADED()
else
	eventFrame:RegisterEvent('VARIABLES_LOADED')
end
